from axelrod import Player

class TitForTat(Player):
    """A player starts by cooperating and then mimics previous move by opponent."""

    name = 'Tit For Tat'

    def strategy(self, opponent):
        try:
            return opponent.history[-1]
        except IndexError:
            return 'C'

class TitFor2Tats(Player):
    """A player starts by cooperating and then defects only after two defects by opponent."""

    name = "Tit For 2 Tats"

    def strategy(self, opponent):
        if opponent.history[-2:] == ['D', 'D']:
            return 'D'
        return 'C'

class TwoTitsForTat(Player):
    """A player starts by cooperating and replies to each defect by two defections."""

    name = "Two Tits For Tat"

    def strategy(self, opponent):
        if 'D' in opponent.history[-2:]:
            return 'D'
        return 'C'

class SuspiciousTitForTat(Player):
    """A player that behaves opposite to Tit For Tat.

    Starts by defecting and then does the opposite of opponent's previous move.
    This the opposite of TIT FOR TAT, also sometimes called BULLY.
    """

    name = "Suspicious Tit For Tat"

    def strategy(self, opponent):
        return 'C' if opponent.history[-1:] == ['D'] else 'D'


class SneakyTitForTat(Player):
    """Tries defecting once and repents if punished."""

    name = "Sneaky Tit For Tat"

    def strategy(self, opponent):
        if len(self.history) < 2:
            return "C"
        if 'D' not in opponent.history:
            return 'D'
        if opponent.history[-1] == 'D' and self.history[-2] == 'D':
            return "C"
        return opponent.history[-1]


"""Tests for mind controllers and other wizards."""

import axelrod

from test_player import TestPlayer


class TestMindController(TestPlayer):

    name = 'Mind Controller'
    player = axelrod.MindController
    stochastic = False

    def test_strategy(self):
        """ Will always make opponent cooperate """

        P1 = axelrod.MindController()
        P2 = axelrod.Cooperator()
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P2.strategy(P1), 'C')

    def test_vs_defect(self):
        """ Will force even defector to cooperate """

        P1 = axelrod.MindController()
        P2 = axelrod.Defector()
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P2.strategy(P1), 'C')

    def test_vs_grudger(self):
        """ Will force even Grudger to forget its grudges"""

        P1 = axelrod.MindController()
        P2 = axelrod.Grudger()
        P1.history = ['D','D','D','D',]
        self.assertEqual(P1.strategy(P2), 'D')
        self.assertEqual(P2.strategy(P1), 'C')

    def test_init(self):
        """Test to make sure parameters are initialised correctly """

        P1 = axelrod.MindController()
        self.assertEqual(P1.history, [])

    def test_reset(self):
        """ test for the reset method """
        P1 = axelrod.MindController()
        P1.history = ['C', 'D', 'D', 'D']
        P1.reset()
        self.assertEqual(P1.history, [])

class TestMindWarper(TestMindController):

    name = "Mind Warper"
    player = axelrod.MindWarper

class TestMindBender(TestMindController):

    name = "Mind Bender"
    player = axelrod.MindBender


"""Tests for the main tournament class."""

import unittest

import axelrod


class TestTournament(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.game = axelrod.Game()

    def test_initialisation(self):
        P1 = axelrod.Defector()
        P2 = axelrod.Defector()
        P3 = axelrod.Defector()
        tournament = axelrod.Tournament(name='test', players=[P1, P2, P3], processes=4)
        self.assertEqual([str(s) for s in tournament.players], ['Defector', 'Defector', 'Defector'])
        self.assertEqual(tournament.game.score(('C', 'C')), (2, 2))
        self.assertEqual(tournament.turns, 200)
        self.assertEqual(tournament.repetitions, 10)
        self.assertEqual(tournament.name, 'test')
        self.assertEqual(tournament.processes, 4)
        anonymous_tournament = axelrod.Tournament(players=[P1, P2, P3])
        self.assertEqual(anonymous_tournament.name, 'axelrod')


    def test_full_tournament(self):
        """A test to check that tournament runs with all non cheating strategies."""
        strategies = [strategy() for strategy in axelrod.basic_strategies + axelrod.ordinary_strategies]
        tournament = axelrod.Tournament(name='test', players=strategies, game=self.game, turns=500, repetitions=2)
        output_of_tournament = tournament.play().results
        self.assertEqual(type(output_of_tournament), list)
        self.assertEqual(len(output_of_tournament), len(strategies))

    def test_tournament(self):
        """Test tournament."""

        outcome = [
            ('Tit For Tat', [2001, 2001, 2001, 2001, 2001]),
            ('Cooperator', [2200, 2200, 2200, 2200, 2200]),
            ('Defector', [2388, 2388, 2388, 2388, 2388]),
            ('Grudger', [2001, 2001, 2001, 2001, 2001]),
            ('Go By Majority', [2001, 2001, 2001, 2001, 2001]),
        ]
        outcome.sort()

        P1 = axelrod.Cooperator()
        P2 = axelrod.TitForTat()
        P3 = axelrod.Defector()
        P4 = axelrod.Grudger()
        P5 = axelrod.GoByMajority()
        tournament = axelrod.Tournament(name='test', players=[P1, P2, P3, P4, P5], game=self.game, turns=200, repetitions=5)
        names = [str(p) for p in tournament.players]
        results = tournament.play().results
        scores = [[sum([r[i] for ir,r in enumerate(res) if ir != ires]) for i in range(5)] for ires,res in enumerate(results)]
        self.assertEqual(sorted(zip(names, scores)), outcome)


if __name__ == '__main__':
    unittest.main()



import unittest
import axelrod


class TestTournamentManager(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.test_output_directory = './assets/'
        cls.test_with_ecological = True
        cls.test_tournament_name = 'test_tournament'
        cls.test_file_name = 'test_file_name'
        cls.test_file_extenstion = 'png'
        cls.test_strategies = [axelrod.Defector, axelrod.Cooperator]
        cls.test_players = [axelrod.Defector(), axelrod.Cooperator()]

        cls.expected_output_file_path = './assets/test_file_name.png'

    def test_init(self):
        mgr = axelrod.TournamentManager(
            self.test_output_directory,
            self.test_with_ecological)
        self.assertEqual(mgr.output_directory, self.test_output_directory)
        self.assertEqual(mgr.tournaments, [])
        self.assertEqual(mgr.with_ecological, self.test_with_ecological)
        self.assertTrue(mgr.pass_cache)

    def test_one_player_per_strategy(self):
        mgr = axelrod.TournamentManager(
            self.test_output_directory,
            self.test_with_ecological)
        players = mgr.one_player_per_strategy(self.test_strategies)
        self.assertIsInstance(players[0], axelrod.Defector)
        self.assertIsInstance(players[1], axelrod.Cooperator)

    def test_output_file_path(self):
        mgr = axelrod.TournamentManager(
            self.test_output_directory,
            self.test_with_ecological)
        output_file_path = mgr.output_file_path(
            self.test_file_name, self.test_file_extenstion)
        self.assertEqual(output_file_path, self.expected_output_file_path)

    def test_add_tournament(self):
        mgr = axelrod.TournamentManager(
            self.test_output_directory,
            self.test_with_ecological)
        mgr.add_tournament(
            players=self.test_players, name=self.test_tournament_name)
        self.assertEqual(len(mgr.tournaments), 1)
        self.assertIsInstance(mgr.tournaments[0], axelrod.Tournament)
        self.assertEqual(mgr.tournaments[0].name, self.test_tournament_name)

    def test_valid_cache(self):
        mgr = axelrod.TournamentManager(
            output_directory=self.test_output_directory,
            with_ecological=self.test_with_ecological, load_cache=False)
        mgr.add_tournament(
                players=self.test_players, name=self.test_tournament_name)
        self.assertTrue(mgr.valid_cache(200))
        mgr.deterministic_cache['test_key'] = 'test_value'
        self.assertFalse(mgr.valid_cache(200))
        mgr.cache_valid_for_turns = 500
        self.assertFalse(mgr.valid_cache(200))
        self.assertTrue(mgr.valid_cache(500))



"""Test for the go by majority strategy."""

import axelrod

from test_player import TestPlayer


class TestGoByMajority(TestPlayer):

    name = "Go By Majority"
    player = axelrod.GoByMajority
    stochastic = False

    def test_initial_strategy(self):
        """
        Starts by cooperating
        """
        P1 = axelrod.GoByMajority()
        P2 = axelrod.Player()
        self.assertEqual(P1.strategy(P2), 'C')

    def test_strategy(self):
        """
        If opponent cooperates at least as often as they defect then the player cooperates
        """
        P1 = axelrod.GoByMajority()
        P2 = axelrod.Player()
        P1.history = ['C', 'D', 'D', 'D']
        P2.history = ['D', 'D', 'C', 'C']
        self.assertEqual(P1.strategy(P2), 'C')
        P1.history = ['C', 'C', 'D', 'D', 'D']
        P2.history = ['D', 'D', 'C', 'C', 'D']
        self.assertEqual(P1.strategy(P2), 'D')


def factory_TestGoByRecentMajority(L):

    class TestGoByRecentMajority(TestPlayer):

        name = "Go By Majority/%i" % L
        player = getattr(axelrod, 'GoByMajority%i' % L)
        stochastic = False

        def test_initial_strategy(self):
            """Starts by cooperating."""
            P1 = self.player()
            P2 = axelrod.Player()
            self.assertEqual(P1.strategy(P2), 'C')

        def test_strategy(self):
            """If opponent cooperates at least as often as they defect then the player cooperates."""
            P1 = self.player()
            P2 = axelrod.Player()
            P1.history = ['D'] * int(1.5*L)
            P2.history = ['D'] * (L-1) + ['C'] * (L//2 + 1)
            self.assertEqual(P1.strategy(P2), 'C')
            P1.history = ['C'] * int(1.5*L)
            P2.history = ['C'] * (L-1) + ['D'] * (L//2 + 1)
            self.assertEqual(P1.strategy(P2), 'D')

    return TestGoByRecentMajority

TestGoByMajority5 = factory_TestGoByRecentMajority(5)
TestGoByMajority10 = factory_TestGoByRecentMajority(10)
TestGoByMajority20 = factory_TestGoByRecentMajority(20)
TestGoByMajority40 = factory_TestGoByRecentMajority(40)



from axelrod import Player

class Appeaser(Player):
    """A player who tries to guess what the opponent wants.
    
    Switch the behaviour every time the opponent plays 'D'.
    Start with 'C', switch between 'C' and 'D' when opponent plays 'D'.
    """

    name = 'Appeaser'

    def strategy(self, opponent):
        if len(self.history) == 0:
            self.str = 'C'
        else:
            if opponent.history[-1] == 'D':
                if self.str == 'C':
                    self.str = 'D'
                else:
                    self.str = 'C'
        return self.str
